# -*- coding: utf-8 -*-
try:
        from urlparse import urlparse
except ImportError:
        from urllib.parse import urlparse

import logging
import time

from requests.auth import AuthBase

from .utils import (f5_login, f5_exchange_token)
from .exceptions import F5AuthenticationError

log = logging.getLogger(__name__)

X_F5_AUTH_TOKEN_HEADER = 'X-F5-Auth-Token'
ACCESS_TOKEN_TIMEOUT_SECS = 5*60 # 5 minutes
REFRESH_TOKEN_TIMEOUT_SECS = 10*60*60 # 10 hours


class XF5Auth(AuthBase):
    """Signs the request using an X-F5-Auth-Token"""

    def __init__(self, username=None,
        password=None,
        loginProviderName=None,
        loginReference=None,
        access_token=None,
        refresh_token=None,
        **kwargs):

        self.username=username
        self.password=password
        self.loginProviderName=loginProviderName
        self.loginReference=loginReference
        self.access_token=access_token
        self.refresh_token=refresh_token

        if self.access_token:
            self.access_retrieve_time = time.time()
        else:
            self.access_retrieve_time = 0

        if self.refresh_token:
            self.refresh_retrieve_time = time.time()
        else:
            self.refresh_retrieve_time = 0

    # expiration tracking can be improved if we use JWT
    # but avoiding it now for simplicity and
    # to avoid another dependency
    def _is_refresh_expired(self):
        return time.time() - self.refresh_retrieve_time > REFRESH_TOKEN_TIMEOUT_SECS


    def _is_access_expired(self):
        return time.time() - self.access_retrieve_time > ACCESS_TOKEN_TIMEOUT_SECS


    def __call__(self, r):
        """Send a request using an X-F5-Auth-Token."""

        # if we have don't have an access token...
        if not self.access_token or self._is_access_expired():
            host = urlparse(r.url).hostname
            # use the refresh token to get an access token
            if self.refresh_token and not self._is_refresh_expired():
                self.access_token = f5_exchange_token(host, self.refresh_token)
            # otherwise use the username and password to login
            elif self.username and self.password:
                self.access_token, self.refresh_token = f5_login(
                        host,
                        self.username,
                        self.password,
                        providerName=self.loginProviderName,
                        loginReference=self.loginReference)
            else:
                raise F5AuthenticationError("No valid tokens and no username or password were provided")

        log.debug('Sending request %s using access token %s', r,
                self.access_token)

        headers = r.headers
        headers[X_F5_AUTH_TOKEN_HEADER] = self.access_token

        r.prepare_headers(headers)

        return r

    def __repr__(self):
        return '{0}.{1}(username={2}, loginProviderName={3}, loginReference={4})'.format(
                self.__module__, type(self).__name__,
                self.username, self.loginProviderName, self.loginReference)




# -*- coding: utf-8 -*-
import logging

import requests

from .exceptions import F5AuthenticationError, F5TokenExchangeError

log = logging.getLogger(__name__)

EXCHANGE_PATH = '/mgmt/shared/authn/exchange'
LOGIN_PATH = '/mgmt/shared/authn/login'

def f5_exchange_token(host, refresh_token):

    url = 'https://%s%s' % (host, EXCHANGE_PATH)
    body = {'refreshToken':{'token':refresh_token}}

    log.debug('Sending refresh token exchange request.')
    resp = requests.post(url=url, json=body, verify=False)

    if not resp.ok:
        msg = resp.json().get('message', None)
        errorMsg = "Failed to exchange refresh token."
        if msg:
            errorMsg += '  Reason: %s' % msg
        raise F5AuthenticationError(errorMsg)

    resp_json = resp.json()
    access_token = resp_json['token']['token']
    log.debug('Received access token %s', access_token)
    return access_token



def f5_login(host,
        username,
        password,
        providerName=None,
        loginReference=None):

    url = 'https://%s/%s' % (host, LOGIN_PATH)
    body = {'username': username, 'password':password}

    if providerName:
        body['loginProviderName'] = providerName

    if loginReference:
        body['loginReference'] = loginReference

    log.debug('Sending login request for %s', username)
    resp = requests.post(url=url, json=body, verify=False)

    if not resp.ok:
        msg = resp.json().get('message', None)
        errorMsg = "Failed to login user %s." % username
        if msg:
            errorMsg += '  Reason: %s' % msg
        raise F5AuthenticationError(errorMsg)

    resp_json = resp.json()
    access_token = resp_json['token']['token']
    refresh_token = resp_json['refreshToken']['token']
    log.debug('Received refresh token %s.', refresh_token)
    log.debug('Received access token %s.', access_token)
    return access_token, refresh_token





import logging

from .xf5_auth import XF5Auth

__version__ = '0.1.2'

import requests
if requests.__version__ < '2.0.0':
        msg = ('You are using requests version %s, which is older than'
                'requests-xf5auth expects, please upgrade to 2.0.0 or later.')
        raise Warning(msg % requests.__version__)

        logging.getLogger('requests_oauthlib').addHandler(logging.NullHandler())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import re

from setuptools import setup


# Get the version
version_regex = r'__version__ = ["\']([^"\']*)["\']'
with open('requests_f5auth/__init__.py', 'r') as f:
    text = f.read()
    match = re.search(version_regex, text)

    if match:
        VERSION = match.group(1)
    else:
        raise RuntimeError("No version number found!")


APP_NAME = 'requests-f5auth'
DESCRIPTION = 'F5 REST Authentication support for Requests.'

# Publish Helper.
if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()


setup(
    name=APP_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=open('README.rst').read(),
    author='Ivan Mecimore',
    author_email='imecimore@gmail.com',
    url='https://github.com/requests/requests-f5auth',
    packages=['requests_f5auth'],
    install_requires=['requests>=2.0.0'],
    license='ISC',
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ),
    zip_safe=False,
    tests_require=[
        'nose',
        'nose-testconfig',
        'pyyaml'
    ],
    test_suite='tests'
)



"""
requests_xf5auth.exceptions
~~~~~~~~~~~~~~~~~~~
This module contains the set of exceptions.
"""
from requests.exceptions import RequestException


class F5AuthenticationError(RequestException):
    """F5 Authentication Error"""

class F5TokenExchangeError(RequestException):
    """Refresh Token Exchange Failed Error"""
